package com.aitrainer.api.openai

import com.aallam.openai.client.OpenAI
import com.aallam.openai.api.completion.CompletionRequest
import com.aallam.openai.api.completion.TextCompletion
import com.aallam.openai.api.model.Model
import com.aallam.openai.api.model.ModelId
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.util.Properties

class OpenAIService {


    private var openAI: OpenAI? = null
    var model: Model? = null
    private val properties = Properties()

    init {
        val inputStream = ClassLoader.getSystemResourceAsStream("application.properties")
        properties.load(inputStream)
        inputStream?.close()
    }

    private var modelId: ModelId? = null
    private suspend fun connect(modelName: String) {
        openAI = OpenAI(properties.getProperty("openai.key"))
        modelId = ModelId(modelName)
        model = openAI!!.model(modelId!!)

    }

    suspend fun completion(question: String): String {
        return withContext(Dispatchers.IO) {
            if (openAI == null) {
                connect("text-davinci-003")
            }
            val completionRequest = CompletionRequest(
                model = modelId!!,
                prompt = question,
                //echo = true,
                maxTokens = 128,
                temperature=0.1,
            )
            val completion: TextCompletion = openAI!!.completion(completionRequest)
            val result = completion.choices[0].text

            //println("Completion: $result")
            result
        }
    }
}

