package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.DietRawMaterial
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietRawMaterialTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private val dietRawMaterial = DietRawMaterial(
        dietId = 1,
        rawMaterialId = 10,
        name = "Tükörtojás",
        kcalMin = 70,
        kcalMax = 110,
        proteinMin = 19,
        proteinMax = 25,
        fatMin = 12,
        fatMax = 15,
        chMin = 1,
        chMax = 3,
        sugar = 0,
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert dietRawMaterial successfully`() {

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_raw_material")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietRawMaterial))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietId").value(1))
        .andExpect(jsonPath("$.rawMaterialId").value(10))
        .andExpect(jsonPath("$.proteinMax").value(25))

    }

    @Test
    fun `get dietRawMaterial list by dietId id successfully`() {

        // Act & Assert
        mockMvc.perform(get("/api/diet_raw_material/{dietId}", 1)
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].name").value("Tükörtojás"))

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
