package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.Diet
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietTest {

    @Autowired
    private lateinit var mockMvc: MockMvc


    private val diet = Diet(
        dietUserId = 1,
        dietText = "Test diet text",
        monday = "Monday meal",
        tuesday = "Tuesday meal",
        wednesday = "Wednesday meal",
        thursday = "Thursday meal",
        friday = "Friday meal",
        saturday = "Saturday meal",
        sunday = "Sunday meal"
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert diet successfully`() {
        // Arrange
        //given(dietRepository.save(diet)).willReturn(diet)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(diet))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietUserId").value(1))

    }

    @Test
    fun `get diet list by dietUserId id successfully`() {
        mockMvc.perform(get("/api/diet/{dietUserId}", 1)
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].monday").value("Monday meal"))
    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
