package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.Recipe
import com.aitrainer.api.model.diet.RecipeRawMaterial
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.AfterAll
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class RecipeTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private val recipe1 = Recipe(
        name = "Bableves",
        description = "Hozzávalók: ---- Elkészítés: ---",
        cal = 180,
        protein = 19.0,
        fat = 56.0,
        ch = 23.0,
        dietUserId = 4,
    )
    private val recipe2 = Recipe(
        name = "Mákos tészta",
        description = "Hozzávalók: ---- Elkészítés: ---",
        cal = 220,
        protein = 19.0,
        fat = 30.0,
        ch = 78.0,
        dietUserId = 4,
    )
    private val recipe3 = Recipe(

        name = "Borsóleves",
        description = "Hozzávalók: ---- Elkészítés: ---",
        cal = 160,
        protein = 45.0,
        fat = 30.0,
        ch = 34.0,
        dietUserId = 1,
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }

    @AfterAll
    fun tearDown() {

    }


    @Test
    fun `insert recipes successfully`() {

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/recipe")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(recipe1))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.name").value("Bableves"))
        .andExpect(jsonPath("$.cal").value(180))
        .andExpect(jsonPath("$.dietUserId").value(4))
            .andReturn()

        val gson= Gson()
        val newRecipeJson = mvcResult.response.contentAsString
        val newRecipe = gson.fromJson(newRecipeJson, Recipe::class.java)


        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/recipe")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(recipe2))
        )
            .andExpect(status().isOk)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/recipe")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(recipe3))
        )
            .andExpect(status().isOk)


        val recipeRawMaterial1 = RecipeRawMaterial()
        with(recipeRawMaterial1) {
            quantity = 30
            quantityUnit = "g"
            rawMaterialId = 2
            recipe = recipe1
        }
        newRecipe.rawMaterials.add(recipeRawMaterial1)

        val recipeRawMaterial2 = RecipeRawMaterial()
        with (recipeRawMaterial2) {
            quantity = 50
            quantityUnit = "g"
            rawMaterialId = 3
            recipe = recipe1
        }
        newRecipe.rawMaterials.add(recipeRawMaterial2)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/recipe/"+newRecipe.recipeId)
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(newRecipe))
        ).andExpect(status().isOk)
            .andExpect(jsonPath("$.cal").value(180))
            .andExpect(jsonPath("$.rawMaterials.[0].quantity").value(30))
            .andExpect(jsonPath("$.rawMaterials.[0].rawMaterialId").value(2))
            .andExpect(jsonPath("$.rawMaterials.[1].quantity").value(50))
            .andExpect(jsonPath("$.rawMaterials.[1].rawMaterialId").value(3))

        // Act & Assert
        mockMvc.perform(get("/api/recipe" )
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].name").value("Bableves"))
            .andExpect(jsonPath("$.[1].name").value("Mákos tészta"))
            .andExpect(jsonPath("$.[2].dietUserId").value(1))
            .andExpect(jsonPath("$.[2].name").value("Borsóleves"))
            .andExpect(jsonPath("$.[0].rawMaterials.[0].quantity").value(30))
            .andExpect(jsonPath("$.[0].rawMaterials.[1].quantity").value(50))
            .andExpect(jsonPath("$.[0].rawMaterials.[1].rawMaterialId").value(3))


        // Act & Assert
        mockMvc.perform(get("/api/recipe/4")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].name").value("Bableves"))
            .andExpect(jsonPath("$.[1].name").value("Mákos tészta"))
            .andExpect(jsonPath("$.[0].rawMaterials.[1].rawMaterialId").value(3))


        // Act & Assert
        mockMvc.perform(get("/api/recipe/name/leves")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].name").value("Bableves"))
            .andExpect(jsonPath("$.[1].name").value("Borsóleves"))
            .andExpect(jsonPath("$.[0].rawMaterials.[1].rawMaterialId").value(3))

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
